#include <wayland-client-protocol.h>
#include <gdk/wayland/gdkwayland.h>
#include <assert.h>
#include "launcher.h"
#include "widgets/dashboard-widget.h"
#include "widgets/taskbar-widget.h"
#include "foreign.h"
#include "wayland.h"

struct _DiyaLauncher
{
    DiyaShellObject parent;
    DiyaDashboardWidget* dashboard;
    DiyaTaskbarWidget* taskbar;
};
G_DEFINE_FINAL_TYPE(DiyaLauncher, diya_launcher, DIYA_TYPE_SHELL_OBJECT)

enum
{
    NO_PROP,
    PROP_DASHBOARD,
    PROP_TASKBAR,
    N_PROPERTIES
};

static GParamSpec *g_prop[N_PROPERTIES] = {0};


static void on_foreign_window_change(DiyaSessionShell* shell, DiyaForeignWindow * win, gpointer data)
{
    (void) data;
    assert(win);
    assert(shell);
    g_warning("on_foreign_window_change: WINDOW CHANGE %s, shell %s", diya_object_to_string(DIYA_OBJECT(win)), diya_object_to_string(DIYA_OBJECT(shell)));
}

static void on_foreign_window_removed(DiyaSessionShell* shell, DiyaForeignWindow * win, gpointer data)
{
    (void) data;
    assert(win);
    assert(shell);
    g_warning("on_foreign_window_removed: WINDOW REMOVED %s, shell %s", diya_object_to_string(DIYA_OBJECT(win)), diya_object_to_string(DIYA_OBJECT(shell)));
}


static void show_windows(GtkWidget *widget,gpointer data)
{
    (void) widget;
    g_warning("Show all windows");
    DiyaSessionShell * shell = DIYA_SESSION_SHELL(data);
    assert(shell);
    GHashTable* windows = NULL;
    g_object_get(shell, DIYA_PROP_SESSION_WINDOWS, &windows, NULL);
    assert(windows);
    GHashTableIter iter;
    gpointer handle;
    DiyaForeignWindow* window;
    g_hash_table_iter_init (&iter, windows);
    while (g_hash_table_iter_next(&iter, (gpointer) &handle, (gpointer) &window))
    {
        g_warning("unset minimized %s", diya_object_to_string(window));
        diya_foreign_window_set_state(window, DIYA_WIN_STATE_MINIMIZE, false);
    }
}


static void diya_launcher_dispose(GObject* object)
{
    (void) object;
    DiyaLauncher * self = DIYA_LAUNCHER(object);
    g_debug("diya_launcher_dispose: %s", diya_object_to_string(object));
    if(self->dashboard)
    {
        gtk_window_destroy(GTK_WINDOW(self->dashboard));
    }
    if(self->taskbar)
    {
        gtk_window_destroy(GTK_WINDOW(self->taskbar));
    }
    G_OBJECT_CLASS(diya_launcher_parent_class)->dispose(object);
}

static void diya_launcher_init(DiyaLauncher * self)
{
    self->taskbar = NULL;
    self->dashboard = NULL;
}

static const gchar* diya_launcher_to_string(DiyaObject* object)
{
    (void) object;
    return "DiyaLauncher";
}


static void diya_launcher_set_property(GObject *object, guint property_id, const GValue *value, GParamSpec *pspec)
{
    DiyaLauncher * self = DIYA_LAUNCHER(object);

    switch (property_id)
    {
    case PROP_DASHBOARD:
        self->dashboard = g_value_get_pointer(value);
        break;
    case PROP_TASKBAR:
        self->taskbar = g_value_get_pointer(value);
        break;
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
        break;
    }
}

static void diya_launcher_get_property(GObject *object, guint property_id, GValue *value, GParamSpec *pspec)
{
    DiyaLauncher * self = DIYA_LAUNCHER(object);
    switch (property_id)
    {
    case PROP_DASHBOARD:
        g_value_set_pointer(value, self->dashboard);
        break;
    case PROP_TASKBAR:
        g_value_set_pointer(value, self->taskbar);
        break;
    default:
        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
        break;
    }
}

static void diya_launcher_class_init(DiyaLauncherClass *class)
{
    GObjectClass *gobject_class = G_OBJECT_CLASS(class);
    DiyaObjectClass *base_class = DIYA_OBJECT_CLASS(class);

    gobject_class->dispose = diya_launcher_dispose;
    gobject_class->set_property = diya_launcher_set_property;
    gobject_class->get_property = diya_launcher_get_property;
    base_class->to_string = diya_launcher_to_string;

    g_prop[PROP_DASHBOARD] = g_param_spec_pointer(DIYA_PROP_LAUNCHER_DASHBOARD, NULL, "Launcher dashboard", G_PARAM_READWRITE|G_PARAM_CONSTRUCT_ONLY);
     g_prop[PROP_TASKBAR] = g_param_spec_pointer(DIYA_PROP_LAUNCHER_TASKBAR, NULL, "Launcher taskbar", G_PARAM_READWRITE|G_PARAM_CONSTRUCT_ONLY);
    g_object_class_install_properties (gobject_class, N_PROPERTIES, g_prop);
}

void diya_session_shell_launcher_init(DiyaSessionShell * shell)
{
    assert(shell);
    GtkApplication * app;
    g_object_get(shell, DIYA_PROP_SHELL_APPLICATION, &app, NULL);
    assert(app);

    DiyaDashboardWidget* dashboard = DIYA_DASHBOARD_WIDGET(g_object_new (DIYA_TYPE_DASHBOARD_WIDGET, "application", app, "shell",shell, NULL));
    assert(dashboard);

    DiyaTaskbarWidget* taskbar = DIYA_TASKBAR_WIDGET(g_object_new (DIYA_TYPE_TASKBAR_WIDGET, "application", app, "shell",shell, NULL));
    assert(taskbar);
    //gtk_window_set_transient_for(GTK_WINDOW(dashboard), GTK_WINDOW(taskbar));
    DiyaLauncher* self = DIYA_LAUNCHER(g_object_new (DIYA_TYPE_LAUNCHER, "dashboard", dashboard, "taskbar",taskbar, NULL));
    assert(self);
    g_object_bind_property(taskbar, DIYA_PROP_TASKBAR_SHOW_DASHBOARD, dashboard, DIYA_PROP_DASHBOARD_ACTIVE, G_BINDING_BIDIRECTIONAL);
    g_object_set(shell, "launchpad", self, NULL);
    //g_signal_connect (GTK_WINDOW(dashboard), "destroy", G_CALLBACK (on_launcher_destroy), NULL);


    /*
    GtkWidget *box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 5);
    gtk_box_set_homogeneous (GTK_BOX (box), TRUE);
    gtk_window_set_child (GTK_WINDOW (gtk_window), box);
    gtk_widget_set_halign (box, GTK_ALIGN_CENTER);
    gtk_widget_set_valign (box, GTK_ALIGN_CENTER);
    GtkWidget * button = gtk_button_new_with_label ("lock");
    g_signal_connect (button, "clicked", G_CALLBACK (session_lock), shell);
    gtk_box_append (GTK_BOX (box), button);

    button = gtk_button_new_with_label ("show all windows");
    g_signal_connect (button, "clicked", G_CALLBACK (show_windows), shell);
    gtk_box_append (GTK_BOX (box), button);

    button = gtk_button_new_with_label ("Dashboard");
    g_signal_connect (button, "clicked", G_CALLBACK (show_dashboard), shell);
    gtk_box_append (GTK_BOX (box), button);

    //g_signal_connect (gtk_window, "orientation-changed", G_CALLBACK (on_orientation_changed), NULL);
    */
    
    gtk_window_present (GTK_WINDOW (taskbar));
    
    g_signal_connect (shell, DIYA_SIGNAL_FOREIGN_WINDOW_CHANGED, G_CALLBACK (on_foreign_window_change), NULL);
    g_signal_connect (shell, DIYA_SIGNAL_FOREIGN_WINDOW_REMOVED, G_CALLBACK (on_foreign_window_removed), NULL);

    GList *apps = g_app_info_get_all ();
    GList * l;
    for (l = apps; l != NULL; l = l->next)
    {
        gpointer element_data = l->data;
        
        g_warning("%s", g_app_info_get_display_name(element_data));
        g_warning("%s", g_app_info_get_id(element_data));
        g_warning("%s", g_app_info_get_name(element_data));
        //g_warning("%s", g_app_info_get_icon(element_data));
    }
    g_list_free_full(apps, g_object_unref);
}
